/*******************************************************************************
*    Wecker 1.0.                                                               *
*    Copyright (C) 2010-averyfarwaydate Luca Bertoncello                       *
*    Hartigstrasse, 12 - 01127 Dresden Deutschland                             *
*    E-Mail: lucabert@lucabert.de, lucabert@lucabert.com                       *
*    http://www.lucabert.de/  http://www.lucabert.com/                         *
*                                                                              *
*    Circuit by Luca Bertoncello and Tim Rettmann, Layout by Tim Rettmann      *
*                                                                              *
*    This program is free software; you can redistribute it and/or modify      *
*    it under the terms of the GNU General Public License as published by      *
*    the Free Software Foundation; version 2 of the License.                   *
*                                                                              *
*    This program is distributed in the hope that it will be useful,           *
*    but WITHOUT ANY WARRANTY; without even the implied warranty of            *
*    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             *
*    GNU General Public License for more details.                              *
*                                                                              *
*    You should have received a copy of the GNU General Public License         *
*    along with this program; if not, write to the Free Software               *
*    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA *
*******************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include "inc/wecker.h"
#include "inc/eeprom.h"
#include "inc/dcf77.h"
#include "inc/alarm.h"
#include "inc/card.h"
#include "inc/dogm.h"
#include "inc/button.h"
#include "inc/tools.h"
#include "inc/spi.h"

/**
 * Initialisiert die Weckfunktion
 */
void initAlarm()
{
  bzero(alarmFile, 32);
  alarmTime.seconds = alarmTime.minutes = alarmTime.hours = \
    alarmTime.year = alarmTime.month = alarmTime.day = 0;
  hasToCalculateNextAlarm = FALSE;
}

/**
 * Holt die nächste Weckzeit
 *
 * @param bool startup          Wenn TRUE (Wecker gerade eingeschaltet), prüft ob die nächste Weckzeit
 *                              noch für den aktuellen Tag sein soll (zB: Stromausfall in der Nacht)
 */
void getNextAlarm(bool startup)
{
  spiSetHighSpeed();
  alarmTime.year = currentTime.year;
  alarmTime.month = currentTime.month;
  alarmTime.day = currentTime.day;
  alarmTime.weekDay = currentTime.weekDay;
  bzero(alarmFile, 32);
  if(startup == FALSE)
    changeDay(TYPE_ALARMTIME, DIRECTION_UP);
  setStringToScroll(READINGALARM);
  setScrollLine(2);
  startScroll();
  enableMMC();
  readConfig(TRUE);
  if(startup == TRUE)
  {
    if(alarmTime.hours < currentTime.hours ||
       (alarmTime.hours == currentTime.hours && alarmTime.minutes < currentTime.minutes))
    {
      changeDay(TYPE_ALARMTIME, DIRECTION_UP);
      readConfig(TRUE);
    }
  }
  disableMMC();
  spiSetLowSpeed();
  stopScroll();
}

/**
 * Ändert die nächste Weckzeit
 */
void changeAlarm()
{
  char str[6];
  bool stop = FALSE, currPressVolM, currPressVolP, currAlarm, toDisplay = TRUE;
  uint8_t w;
  long time;

  currPressVolM = currPressVolP = currAlarm = FALSE;
  changeAlarmTime.day = alarmTime.day;
  changeAlarmTime.month = alarmTime.month;
  changeAlarmTime.year = alarmTime.year;
  changeAlarmTime.minutes = alarmTime.minutes;
  changeAlarmTime.hours = alarmTime.hours;
  lcdBacklightOn();
  if(bit_is_clear(BUTTON_PIN, BUTTON_ALARMONOFF))
    currAlarm = TRUE;
  w = 1;
  setStringToScroll(CHANGEALARM1);
  setScrollLine(2);
  startScroll();
  time = getUptime();
  while(!stop)
  {
    if(getUptime() > (time + ALARMBLINK))
    {
      toDisplay = TRUE - toDisplay;
      time = getUptime();
    }
    lcdGotoXY(0, 1);
    if((w == 3 && toDisplay) || (w != 3))
    {
      if(changeAlarmTime.day < 10)
        lcdPutc('0');
      itoa(changeAlarmTime.day, str, 10);
      lcdPuts(str);
      lcdPutc('.');
      if(changeAlarmTime.month < 10)
        lcdPutc('0');
      itoa(changeAlarmTime.month, str, 10);
      lcdPuts(str);
      lcdPutc('.');
      itoa(changeAlarmTime.year + 2000, str, 10);
      lcdPuts(str);
    }
    else
      lcdPuts("          ");
    lcdPutc(' ');
    if((w == 1 && toDisplay) || (w != 1))
    {
      if(changeAlarmTime.hours < 10)
        lcdPutc('0');
      itoa(changeAlarmTime.hours, str, 10);
      lcdPuts(str);
    }
    else
      lcdPuts("  ");
    lcdPutc(':');
    if((w == 2 && toDisplay) || (w != 2))
    {
      if(changeAlarmTime.minutes < 10)
        lcdPutc('0');
      itoa(changeAlarmTime.minutes, str, 10);
      lcdPuts(str);
    }
    else
      lcdPuts("  ");

    if((currAlarm == TRUE) && bit_is_set(BUTTON_PIN, BUTTON_ALARMONOFF))
      currAlarm = FALSE;
    if((currPressVolP == TRUE) && bit_is_set(BUTTON_PIN, BUTTON_VOLPLUS))
      currPressVolP = FALSE;
    if((currPressVolM == TRUE) && bit_is_set(BUTTON_PIN, BUTTON_VOLMINUS))
      currPressVolM = FALSE;

    if((currAlarm == FALSE) && bit_is_clear(BUTTON_PIN, BUTTON_ALARMONOFF))
    { // Wechselt zwischen Minuten, Stunde und Datum
      currAlarm = TRUE;
      w++;
      if(w == 4)
        w = 1;
      switch(w)
      {
        case 1: // Änderung der Stunde
          stopScroll();
          setStringToScroll(CHANGEALARM1);
          startScroll();
        break;
        case 2: // Änderung der Minuten
          stopScroll();
          setStringToScroll(CHANGEALARM2);
          startScroll();
        break;
        case 3: // Änderung des Datums
          stopScroll();
          setStringToScroll(CHANGEALARM3);
          startScroll();
        break;
      }
    }
    if((currPressVolP == FALSE) && bit_is_clear(BUTTON_PIN, BUTTON_VOLPLUS))
    {
      currPressVolP = TRUE;
      switch(w)
      {
        case 1: // Änderung der Stunde
          changeAlarmTime.hours++;
          if(changeAlarmTime.hours == 24)
            changeAlarmTime.hours = 0;
        break;
        case 2: // Änderung der Minuten
          changeAlarmTime.minutes++;
          if(changeAlarmTime.minutes == 60)
            changeAlarmTime.minutes = 0;
        break;
        case 3: // Änderung des Datums
          changeDay(TYPE_CHANGEALARMTIME, DIRECTION_UP);
        break;
      }
    }
    if((currPressVolM == FALSE) && bit_is_clear(BUTTON_PIN, BUTTON_VOLMINUS))
    {
      currPressVolM = TRUE;
      switch(w)
      {
        case 1: // Änderung der Stunde
          changeAlarmTime.hours--;
          if(changeAlarmTime.hours == -1)
            changeAlarmTime.hours = 23;
        break;
        case 2: // Änderung der Minuten
          if(changeAlarmTime.minutes > 0)
            changeAlarmTime.minutes--;
          else
            changeAlarmTime.minutes = 59;
        break;
        case 3: // Änderung des Datums
          if(!(changeAlarmTime.year == 10 && changeAlarmTime.month == 1 && changeAlarmTime.day == 1))
            changeDay(TYPE_CHANGEALARMTIME, DIRECTION_DOWN);
        break;
      }
    }
    if(bit_is_clear(BUTTON_PIN, BUTTON_MENU))
    { // Eingabe wurde bestätigt
      stop = TRUE;
      alarmTime.day = changeAlarmTime.day;
      alarmTime.month = changeAlarmTime.month;
      alarmTime.year = changeAlarmTime.year;
      alarmTime.hours = changeAlarmTime.hours;
      alarmTime.minutes = changeAlarmTime.minutes;
      loop_until_bit_is_set(BUTTON_PIN, BUTTON_MENU);
    }
    if(bit_is_clear(BUTTON_PIN, BUTTON_FORCEDCFSYNC))
    { // Abgebrochen
      stop = TRUE;
      loop_until_bit_is_set(BUTTON_PIN, BUTTON_FORCEDCFSYNC);
    }

    if(hasToScroll())
      doScroll();
  }
  stopScroll();
  lcdClear();
  lcdBacklightOff();
}
