/*******************************************************************************
*    Wecker 1.0.                                                               *
*    Copyright (C) 2010-averyfarwaydate Luca Bertoncello                       *
*    Hartigstrasse, 12 - 01127 Dresden Deutschland                             *
*    E-Mail: lucabert@lucabert.de, lucabert@lucabert.com                       *
*    http://www.lucabert.de/  http://www.lucabert.com/                         *
*                                                                              *
*    Circuit by Luca Bertoncello and Tim Rettmann, Layout by Tim Rettmann      *
*                                                                              *
*    This program is free software; you can redistribute it and/or modify      *
*    it under the terms of the GNU General Public License as published by      *
*    the Free Software Foundation; version 2 of the License.                   *
*                                                                              *
*    This program is distributed in the hope that it will be useful,           *
*    but WITHOUT ANY WARRANTY; without even the implied warranty of            *
*    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             *
*    GNU General Public License for more details.                              *
*                                                                              *
*    You should have received a copy of the GNU General Public License         *
*    along with this program; if not, write to the Free Software               *
*    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA *
*******************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <util/delay.h>
#include "inc/spi.h"
#include "inc/wecker.h"
#include "inc/eeprom.h"
#include "inc/dcf77.h"
#include "inc/dogm.h"
#include "inc/button.h"
#include "inc/mp3.h"
#include "inc/card.h"
#include "inc/alarm.h"
#include "resources/dict.h"

/**
 * Initialisiert die Taster
 */
void initButtons()
{
// Alle Knöpfe sind Eingänge
  BUTTON_DDR &= ~_BV(BUTTON_BACKLIGHT);
  BUTTON_DDR &= ~_BV(BUTTON_ALARMONOFF);
  BUTTON_DDR &= ~_BV(BUTTON_FORCEDCFSYNC);
  BUTTON_DDR &= ~_BV(BUTTON_MENU);
  BUTTON_DDR &= ~_BV(BUTTON_VOLPLUS);
  BUTTON_DDR &= ~_BV(BUTTON_VOLMINUS);
// KEINE Interne Pullups, denn die Entprellung dient schon als Pullup!
  BUTTON_PORT = 0;

  startBacklight = displayMenu = displayLastSync = startPressDCF = startDisplayVolume = displayNextAlarm = startGetNextMP3 = 0L;
  countMenuPress = 0;
  playMP3 = isHoliday = FALSE;
  currPressAlarm = currPressMenu = currPressVolP = currPressVolM = changedVolume = FALSE;
}

/**
 * Verwaltet die Knöpfe
 */
void manageButtons()
{
  if(!isSynchonizing())
  {
    if(!startedMP3())
    {
      if((currPressAlarm == TRUE) && bit_is_set(BUTTON_PIN, BUTTON_ALARMONOFF))
        currPressAlarm = FALSE;
      if((currPressAlarm == FALSE) && bit_is_clear(BUTTON_PIN, BUTTON_ALARMONOFF))
      {
        currPressAlarm = TRUE;
        if(displayNextAlarm != 0L)
        {
          changeAlarm();
          displayMenu = displayNextAlarm = 0L;
          countMenuPress = 0;
        }
        else
        {
          if(bit_is_clear(BUTTON_PIN, BUTTON_MENU))
          { // Wenn die Taste gedruckt wird, und die Menütaste gedruckt bleibt, spielt die Test-MP3 ab
            if(sd_raw_available())
            {
              setNextFileToPlay(MP3DIRTEST, mp3TestFile);
              playMP3();
            }
            displayMenu = 0L;
            countMenuPress = 0;
          }
          else
          {
            if(isAlarmOn())
            {
              setAlarmOff();
            }
            else
            {
              setAlarmOn();
            }
            setToDisplay();
          }
        }
      }

      if(displayHoliday)
      {
        if(bit_is_clear(BUTTON_PIN, BUTTON_FORCEDCFSYNC))
        {
          isHoliday = TRUE - isHoliday;
          loop_until_bit_is_set(BUTTON_PIN, BUTTON_FORCEDCFSYNC);
          displayMenu = 0L;
          displayNextAlarm = 0L;
          displayHoliday = FALSE;
          countMenuPress = 0;
          getNextAlarm(FALSE);
        }
      }
      else
      {
        if((startGetNextMP3 == 0L) && (startPressDCF == 0L) && bit_is_clear(BUTTON_PIN, BUTTON_FORCEDCFSYNC))
        {
          startPressDCF = getUptime();
          displayLastSync = getUptime();
        }
        if((startGetNextMP3 == 0L) && (startPressDCF != 0L) && bit_is_clear(BUTTON_PIN, BUTTON_FORCEDCFSYNC) && ((getUptime() - startPressDCF) >= PRESSTOFORCEDCF))
        {
          startPressDCF = 0L;
          displayLastSync = 0L;
          startSync();
        }
      }

      if((currPressMenu == TRUE) && bit_is_set(BUTTON_PIN, BUTTON_MENU))
        currPressMenu = FALSE;
      if((currPressMenu == FALSE) && bit_is_clear(BUTTON_PIN, BUTTON_MENU))
      {
        currPressMenu = TRUE;
        countMenuPress++;
        if(countMenuPress == 1)
          displayMenu = getUptime();      // Einmal drucken zeigt die Informationen über das Programm
        if(countMenuPress == 2)
        {                                 // Zweimal drucken zeigt die nächste Weckzeit
          displayMenu = 0L;
          displayNextAlarm = getUptime();
        }
        if(countMenuPress == 3)
        {                                 // Dreimal drucken gibt die Möglichkeit Urlaub zu setzen
          displayNextAlarm = getUptime();
          displayHoliday = TRUE;
        }
        if(countMenuPress == 4)
        {                                 // Viermal drucken zeigt die aktuelle Lautstärke
          displayMenu = 0L;
          displayNextAlarm = 0L;
          displayHoliday = FALSE;
          displayCurrentVolume();
          loop_until_bit_is_set(BUTTON_PIN, BUTTON_MENU);
        }
        if(countMenuPress == 5)
        {                                 // Fünfmal drucken zeigt das nächstgespieltes Stück
          lcdGotoXY(0, 2);
          lcdPuts(CLEARLINE);
          displayMenu = 0L;
          displayNextAlarm = 0L;
          getActualMP3();                 // Holt das aktuelle Stück
          setStringToScroll(mp3ToPlay);
          lcdGotoXY(0, 0);
          lcdPuts(NEXTMP3);
          setScrollLine(1);
          startScroll();
          startGetNextMP3 = getUptime();
          maxDisplayNextMP3 = (strlen(mp3ToPlay) + 3) * SCROLLINTERVALL;
        }
        if(countMenuPress == 6)
        {                                 // Sechsmal drucken geht zurück zur Zeitanzeige
          displayMenu = 0L;
          displayNextAlarm = 0L;
          displayHoliday = FALSE;
          countMenuPress = 0;
          lcdGotoXY(0, 2);
          lcdPuts(CLEARLINE);
        }
      }
    }

    if((currPressVolP == TRUE) && bit_is_set(BUTTON_PIN, BUTTON_VOLPLUS))
      currPressVolP = FALSE;
    if((currPressVolP == FALSE) && bit_is_clear(BUTTON_PIN, BUTTON_VOLPLUS))
    {
      currPressVolP = TRUE;
      if(bit_is_clear(BUTTON_PIN, BUTTON_MENU))
      { // Menü/+, holen die nächste MP3
        if(sd_raw_available())
        {
          loop_until_bit_is_set(BUTTON_PIN, BUTTON_MENU);
          getNextMP3();
          setStringToScroll(mp3ToPlay);
          setScrollLine(2);
          startScroll();
          startGetNextMP3 = getUptime();
          maxDisplayNextMP3 = (strlen(mp3ToPlay) + 3) * SCROLLINTERVALL;
          displayMenu = 0L;
          lcdBacklightOn();
        }
      }
      else if(currentVolume > 0x00)
      {
        currentVolume -= 0x05;
        if(startedMP3()) // Die Lautstärke wird nur geändert, wenn gerade eine MP3 abgespielt wird
          vsSetVolume(currentVolume);
        changedVolume = TRUE;
        displayCurrentVolume();
      }
    }

    if((currPressVolM == TRUE) && bit_is_set(BUTTON_PIN, BUTTON_VOLMINUS))
      currPressVolM = FALSE;
    if((currPressVolM == FALSE) && bit_is_clear(BUTTON_PIN, BUTTON_VOLMINUS))
    {
      currPressVolM = TRUE;
      if(bit_is_clear(BUTTON_PIN, BUTTON_MENU))
      { // Menü/-, holen die vorige MP3, wenn gerade nicht schon am Anfang der Liste ist
        if(sd_raw_available())
        {
          if(getCurrentFileNumber() > 0)
          {
            loop_until_bit_is_set(BUTTON_PIN, BUTTON_MENU);
            setCurrentFileNumber(getCurrentFileNumber() - 1);
            getNextMP3();
            setCurrentFileNumber(getCurrentFileNumber() - 1);
            setStringToScroll(mp3ToPlay);
            setScrollLine(2);
            startScroll();
            startGetNextMP3 = getUptime();
            maxDisplayNextMP3 = (strlen(mp3ToPlay) + 3) * SCROLLINTERVALL;
            displayMenu = 0L;
            lcdBacklightOn();
          }
        }
      }
      else if(currentVolume < 0xFA)
      {
        currentVolume += 0x05;
        if(startedMP3()) // Die Lautstärke wird nur geändert, wenn gerade eine MP3 abgespielt wird
          vsSetVolume(currentVolume);
        changedVolume = TRUE;
        displayCurrentVolume();
      }
    }
  }

  if(!startedMP3())
  {
// Die Hintergrundbeleuchtung soll auch bei der Synchronisierung funktionieren
    if(bit_is_clear(BUTTON_PIN, BUTTON_BACKLIGHT))
    {
      lcdBacklightOn();
      startBacklight = getUptime();
    }

// Timeouts für die Funktionen
    if(startBacklight != 0L && (getUptime() - startBacklight) >= BACKLIGHTTIME)
    {
      lcdBacklightOff();
      startBacklight = 0;
    }

    if(displayMenu != 0L && (getUptime() - displayMenu) >= DISPLAYMENUTIME)
    {
      displayMenu = 0L;
      countMenuPress = 0;
      lcdGotoXY(0, 2);
      lcdPuts(CLEARLINE);
    }

    if(displayNextAlarm != 0L && (getUptime() - displayNextAlarm) >= DISPLAYNEXTALARMTIME)
    {
      displayNextAlarm = 0L;
      countMenuPress = 0;
      displayHoliday = FALSE;
      lcdGotoXY(0, 2);
      lcdPuts(CLEARLINE);
    }

    if(displayLastSync != 0L && bit_is_set(BUTTON_PIN, BUTTON_FORCEDCFSYNC) && (getUptime() - displayLastSync) >= DISPLAYLASTSYNCTIME)
    {
      displayLastSync = 0L;
      startPressDCF = 0L;
    }
  }
}

/**
 * Zeigt die aktuelle Lautstärke
 * Die Lautstärke ist zwischen 0 und 15
 */
void displayCurrentVolume()
{
  char str[5];
  uint8_t vol;

  vol = 50 - (currentVolume / 5);
  lcdGotoXY(0, 2);
  lcdPuts(CURRENTVOLUME);
  lcdGotoXY(13, 2);
  if(vol < 10)
    lcdPutc(' ');
  itoa(vol, str, 10);
  lcdPuts(str);
  startDisplayVolume = getUptime();
}
