/*******************************************************************************
*    Wecker 1.0.                                                               *
*    Copyright (C) 2010-averyfarwaydate Luca Bertoncello                       *
*    Hartigstrasse, 12 - 01127 Dresden Deutschland                             *
*    E-Mail: lucabert@lucabert.de, lucabert@lucabert.com                       *
*    http://www.lucabert.de/  http://www.lucabert.com/                         *
*                                                                              *
*    Circuit by Luca Bertoncello and Tim Rettmann, Layout by Tim Rettmann      *
*                                                                              *
*    This program is free software; you can redistribute it and/or modify      *
*    it under the terms of the GNU General Public License as published by      *
*    the Free Software Foundation; version 2 of the License.                   *
*                                                                              *
*    This program is distributed in the hope that it will be useful,           *
*    but WITHOUT ANY WARRANTY; without even the implied warranty of            *
*    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             *
*    GNU General Public License for more details.                              *
*                                                                              *
*    You should have received a copy of the GNU General Public License         *
*    along with this program; if not, write to the Free Software               *
*    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA *
*******************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <util/delay.h>
#include "inc/wecker.h"
#include "inc/dogm.h"
#include "inc/tools.h"
#include "inc/button.h"
#include "inc/card.h"
#include "inc/spi.h"
#include "inc/mp3.h"
#include "inc/dcf77.h"
#include "inc/alarm.h"
#include "inc/time.h"
#include "resources/dict.h"

/**
 * Initialisiert die MMC-Karte.
 * Falls gerade keine Karte eingelegt ist, wartet, bis eine Karte eingelegt wird
 *
 * @param bool displayInsertCard    TRUE wenn die Meldung "Karte einlegen" angezeigt werden soll
 */
void initCard(bool displayInsertCard)
{
  MMCALLOW_DDR |= _BV(MMCALLOW_PIN);

  do
  {
    if(displayInsertCard)
    {
      lcdBacklightOn();
      lcdGotoXY(0, 0);
      lcdPuts(INSERTCARD);
    }
    else
      _delay_ms(50);
  } while(!sd_raw_init());
  lcdBacklightOff();
  _delay_ms(100);

  partition = partition_open(sd_raw_read,
                             sd_raw_read_interval,
                             0,
                             0,
                             0);
  if(!partition)
  {
    partition = partition_open(sd_raw_read,
                             sd_raw_read_interval,
                             0,
                             0,
                             -1
                            );
    if(!partition)
    {
      lcdGotoXY(0, 0);
      lcdPuts(FAILEDTOINITFAT);
      lcdGotoXY(0, 1);
      lcdPuts("Err: 001        ");
      lcdGotoXY(0, 2);
      lcdPuts(CLEARLINE);
      while(1)
        ;
    }
  }

  fs = fat_open(partition);
  if(!fs)
  {
    lcdGotoXY(0, 0);
    lcdPuts(FAILEDTOINITFAT);
    lcdGotoXY(0, 1);
    lcdPuts("Err: 002        ");
    lcdGotoXY(0, 2);
    lcdPuts(CLEARLINE);
    while(1)
      ;
  }

  fat_get_dir_entry_of_path(fs, "/", &directory);
  dd = fat_open_dir(fs, &directory);
  if(!dd)
  {
    lcdGotoXY(0, 0);
    lcdPuts(FAILEDTOINITFAT);
    lcdGotoXY(0, 1);
    lcdPuts("Err: 003        ");
    lcdGotoXY(0, 2);
    lcdPuts(CLEARLINE);
    while(1)
      ;
  }
  cardToBeInserted = FALSE;
  waitingForCard = 0L;
}


/**
 * Welchselt zu einer Directory
 *
 * @param char *dir                 dir Directory zu wechseln
 * @return bool                     TRUE wenn erfolgreich gewechselt, FALSE ansonsten
 */
bool chDir(char *dir)
{
  struct fat_dir_entry_struct subdir_entry;
  struct fat_dir_struct* dd_new;

  if(strcmp(dir, "/") == 0)
  {
    fat_close_dir(dd);
    fat_get_dir_entry_of_path(fs, "/", &directory);
    dd_new = fat_open_dir(fs, &directory);
    if(dd_new)
    {
      dd = dd_new;
      return TRUE;
    }
    return FALSE;
  }
  while(fat_read_dir(dd, &subdir_entry))
  {
    if(strcmp(subdir_entry.long_name, dir) == 0)
    {
      fat_reset_dir(dd);
      dd_new = fat_open_dir(fs, &subdir_entry);
      if(dd_new)
      {
        fat_close_dir(dd);
        dd = dd_new;
        return TRUE;
      }
      else
        return FALSE;
    }
  }
  return FALSE;
}

/**
 * Öffnet eine Datei
 *
 * @param const char *name          Der Dateiname
 * @return struct fat_file_struct*  Die Dateistruktur oder 0 wenn nicht möglich
 */
struct fat_file_struct* fopen(const char* name)
{
  struct fat_dir_entry_struct file_entry;

  while(fat_read_dir(dd, &file_entry))
  {
    if(strcmp(file_entry.long_name, name) == 0)
    {
      fat_reset_dir(dd);
      return fat_open_file(fs, &file_entry);
    }
  }

  return 0;
}

/**
 * Verwaltet die SD-Karte
 * Wenn rausgenommen, zeigt eine Fehlermeldung,
 * wenn wieder reingenommen, initialisert sie und liest die
 * Konfigurationsdatei erneut
 */
void manageCard()
{
  if(!sd_raw_available() && !cardToBeInserted)
  {
    fat_close_dir(dd);
    fat_close(fs);
    partition_close(partition);

    cardToBeInserted = TRUE;
    lcdBacklightOn();
    lcdGotoXY(0, 2);
    lcdPuts(INSERTCARD);
    waitingForCard = 0L;
  }
  if(cardToBeInserted && sd_raw_available())
  {
    waitingForCard = getUptime();
    cardToBeInserted = FALSE;
  }
// Nachdem die Karte eingelegt wird, warten wir lieber einen Moment,
// um sicher zu sein, daß die Kontakte sich nicht mehr bewegen
  if(waitingForCard != 0L && sd_raw_available() && (getUptime() > (waitingForCard + WAITINGFORCARD)))
  {
    waitingForCard = 0L;
    enableMMC();
    initCard(FALSE);
    readConfig(FALSE);
    disableMMC();
    lcdBacklightOff();
    lcdGotoXY(0, 2);
    lcdPuts(CLEARLINE);
  }
}

/**
 * Lese eine Zeile. Leere Zeilen, sowie Zeichen ab '#' werden ignoriert
 *
 * @param struct fat_file_struct* fd  Filezeiger
 * @param char* str                   Die String, wo die Daten gespeichert werden
 * @param uint8_t maxlen              Maximale Länge der String
 * @return uint8_t                    VALIDLINE wenn die Zeile gültig ist, EMPTYLINE wenn nicht (leere Zeile, Kommentare, usw.)
 */
uint8_t getLine(struct fat_file_struct* fd, char *str, uint8_t maxlen)
{
  unsigned char c[2];
  uint8_t l, stop;

  bzero(str, maxlen);
  l = 0;
  stop = 0;
  fat_read_file(fd, c, 1);

  if(fat_feof(fd))
    return EMPTYLINE;
  if(c[0] == '#')
  {
    while(c[0] != '\n' && !fat_feof(fd))
      fat_read_file(fd, c, 1);
    return EMPTYLINE;
  }
  if(c[0] == '\n' || c[0] == '\r')
    return EMPTYLINE;
  str[l++] = c[0];
  while(c[0] != '\n' && !fat_feof(fd) && l < maxlen)
  {
    fat_read_file(fd, c, 1);
    if(c[0] == '#')
      stop = 1;
    if(c[0] != '\n' && c[0] != '\r' && !stop)
      str[l++] = c[0];
  }
  while(str[l - 1] == ' ')
    str[--l - 1] = 0;
  return VALIDLINE;
}

/**
 * Lese die Konfigurationsdatei aus der Karte
 *
 * @param bool readAlarm              Wenn TRUE wird die Weckzeit für den Tag in alarmTime gelesen
 *                                    Wenn FALSE wird die allgemeine Konfiguration bewertet
 */
void readConfig(bool readAlarm)
{
  struct fat_file_struct* fd;
  char str[50];
  uint8_t w;
  char days[7][4] = {"SUN", "MON", "TUE", "WED", "THU", "FRI", "SAT"};
  char date[5], alarm[5], searchAlarm[11], extraAlarm[17], extraFile[16];

  if(readAlarm)
  {
    bzero(alarm, 5);
    bzero(alarmFile, 32);
    bzero(searchAlarm, 11);
    bzero(extraAlarm, 17);
    bzero(extraFile, 16);
    strcpy(searchAlarm, "ALARM_");        // Suchstring für die normalen Weckzeiten
    searchAlarm[6] = days[alarmTime.weekDay][0];
    searchAlarm[7] = days[alarmTime.weekDay][1];
    searchAlarm[8] = days[alarmTime.weekDay][2];
    searchAlarm[9] = '=';
    bzero(date, 5);
    itoa(alarmTime.day, str, 10);
    w = 0;
    if(alarmTime.day < 10)
    {
      date[w++] = '0';
      date[w++] = str[0];
    }
    else
    {
      date[w++] = str[0];
      date[w++] = str[1];
    }
    itoa(alarmTime.month, str, 10);
    if(alarmTime.month < 10)
    {
      date[w++] = '0';
      date[w++] = str[0];
    }
    else
    {
      date[w++] = str[0];
      date[w++] = str[1];
    }
    strcpy(extraAlarm, "EXTRAALARM_");    // Suchstring für die extra Weckzeiten
    extraAlarm[11] = date[0];
    extraAlarm[12] = date[1];
    extraAlarm[13] = date[2];
    extraAlarm[14] = date[3];
    extraAlarm[15] = '=';
    strcpy(extraFile, "EXTRAFILE_");      // Suchstring für die MP3 der extra Weckzeiten
    extraFile[10] = date[0];
    extraFile[11] = date[1];
    extraFile[12] = date[2];
    extraFile[13] = date[3];
    extraFile[14] = '=';
  }
  else
  {
    lcdBacklightOn();
    lcdClear();
    lcdGotoXY(0, 0);
    lcdPuts(CLEARLINE);
    lcdGotoXY(0, 1);
    lcdPuts(READINGCONFIG);
  }

  chDir("/");
  fd = fopen(CONFIGFILE);
  if(!fd)
  {
    displayError(ERROROPENCONFIG);
    return;
  }
  w = 15;
  while(!fat_feof(fd))
  {
    if(getLine(fd, str, 50) == VALIDLINE)
    {
      if(readAlarm)
      { // Die Weckzeit wird gelesen
        if(isHoliday)
        {
          if(strncmp(str, "ALARM_HOLIDAY=", 13) == 0)
            strcpy(alarm, str + 14);
        }
        else
        {
          if(strncmp(str, searchAlarm, 9) == 0)
            strcpy(alarm, str + 10);
          if(strncmp(str, extraAlarm, 15) == 0)
            strcpy(alarm, str + 16);
        }
        if(strncmp(str, extraFile, 14) == 0)
          strcpy(alarmFile, str + 15);
        displayCurrentTime();
      }
      else
      { // Die Konfiguration wird gelesen
        lcdGotoXY(w, 2);
        lcdPutc('<');
        lcdPutc(' ');
        if(w > 0)
          w--;
        else
        {
          lcdGotoXY(0, 2);
          lcdPutc(' ');
          w = 15;
        }
        if((strncmp(str, "STARTVOLUME=", 12) == 0) && (changedVolume == FALSE))
        {
          currentVolume = (50 - atoi(str + 12)) * 5;
        }
        if(strncmp(str, "TIMEZONE=", 9) == 0)
        {
          setTimezone(atoi(str + 9));
        }
        if(strncmp(str, "TESTMP3=", 8) == 0)
        {
          strcpy(mp3TestFile, (str + 8));
        }
      }
    }
  }
  fclose(fd);
  if(readAlarm)
  { // Die Weckzeit wird konvertiert
    alarmTime.hours = (alarm[0] - '0') * 10 + (alarm[1] - '0');
    alarmTime.minutes = (alarm[2] - '0') * 10 + (alarm[3] - '0');
  }
  else
    lcdBacklightOff();
}
