/*******************************************************************************
*    Wecker 1.0.                                                               *
*    Copyright (C) 2010-averyfarwaydate Luca Bertoncello                       *
*    Hartigstrasse, 12 - 01127 Dresden Deutschland                             *
*    E-Mail: lucabert@lucabert.de, lucabert@lucabert.com                       *
*    http://www.lucabert.de/  http://www.lucabert.com/                         *
*                                                                              *
*    Circuit by Luca Bertoncello and Tim Rettmann, Layout by Tim Rettmann      *
*                                                                              *
*    This program is free software; you can redistribute it and/or modify      *
*    it under the terms of the GNU General Public License as published by      *
*    the Free Software Foundation; version 2 of the License.                   *
*                                                                              *
*    This program is distributed in the hope that it will be useful,           *
*    but WITHOUT ANY WARRANTY; without even the implied warranty of            *
*    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             *
*    GNU General Public License for more details.                              *
*                                                                              *
*    You should have received a copy of the GNU General Public License         *
*    along with this program; if not, write to the Free Software               *
*    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA *
*******************************************************************************/

#ifndef CARD_H_
#define CARD_H_

#include "../sd/fat.h"
#include "../sd/fat_config.h"
#include "../sd/partition.h"
#include "../sd/sd_raw.h"
#include "../sd/sd_raw_config.h"
#include "../resources/dict.h"

#define WAITINGFORCARD        1000   // Eine Sekunde Wartezeit, wenn die Karte eingelegt wird

#define VALIDLINE             0
#define EMPTYLINE             1

#ifdef SOURCE_card
  struct partition_struct* partition;
  struct fat_fs_struct* fs;
  struct fat_dir_struct* dd;
  struct fat_dir_entry_struct directory;
  bool cardToBeInserted;
  volatile unsigned long waitingForCard;
#else
  extern struct partition_struct* partition;
  extern struct fat_fs_struct* fs;
  extern struct fat_dir_struct* dd;
  extern struct fat_dir_entry_struct directory;
#endif

/**
 * Initialisiert die MMC-Karte.
 * Falls gerade keine Karte eingelegt ist, wartet, bis eine Karte eingelegt wird
 *
 * @param bool displayInsertCard      TRUE wenn die Meldung "Karte einlegen" angezeigt werden soll
 */
void initCard(bool displayInsertCard);

/**
 * Welchselt zu einer Directory
 *
 * @param char *dir                   dir Directory zu wechseln
 * @return bool                       TRUE wenn erfolgreich gewechselt, FALSE ansonsten
 */
bool chDir(char *dir);

/**
 * Öffnet eine Datei
 *
 * @param const char *name            Der Dateiname
 * @return struct fat_file_struct*    Die Dateistruktur oder 0 wenn nicht möglich
 */
struct fat_file_struct* fopen(const char* name);

/**
 * Schließt die Datei.
 * Diese Alias ist nur deswegen da, daß man mit fopen/fclose arbeiten kann
 */
#define fclose(__fd)                fat_close_file(__fd)

/**
 * Verwaltet die SD-Karte
 * Wenn rausgenommen, zeigt eine Fehlermeldung,
 * wenn wieder reingenommen, initialisert sie und liest die
 * Konfigurationsdatei erneut
 */
void manageCard(void);

/**
 * Lese eine Zeile. Leere Zeilen, sowie Zeichen ab '#' werden ignoriert
 *
 * @param struct fat_file_struct* fd  Filezeiger
 * @param char* str                   Die String, wo die Daten gespeichert werden
 * @param uint8_t maxlen              Maximale Länge der String
 * @return uint8_t                    VALIDLINE wenn die Zeile gültig ist, EMPTYLINE wenn nicht (leere Zeile, Kommentare, usw.)
 */
uint8_t getLine(struct fat_file_struct* fd, char *str, uint8_t maxlen);

/**
 * Lese die Konfigurationsdatei aus der Karte
 *
 * @param bool readAlarm              Wenn TRUE wird die Weckzeit für den Tag in alarmTime gelesen
 *                                    Wenn FALSE wird die allgemeine Konfiguration bewertet
 */
void readConfig(bool readAlarm);

#endif /* CARD_H_ */
