/*******************************************************************************
*    Wecker 1.0.                                                               *
*    Copyright (C) 2010-averyfarwaydate Luca Bertoncello                       *
*    Hartigstrasse, 12 - 01127 Dresden Deutschland                             *
*    E-Mail: lucabert@lucabert.de, lucabert@lucabert.com                       *
*    http://www.lucabert.de/  http://www.lucabert.com/                         *
*                                                                              *
*    Circuit by Luca Bertoncello and Tim Rettmann, Layout by Tim Rettmann      *
*                                                                              *
*    This program is free software; you can redistribute it and/or modify      *
*    it under the terms of the GNU General Public License as published by      *
*    the Free Software Foundation; version 2 of the License.                   *
*                                                                              *
*    This program is distributed in the hope that it will be useful,           *
*    but WITHOUT ANY WARRANTY; without even the implied warranty of            *
*    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             *
*    GNU General Public License for more details.                              *
*                                                                              *
*    You should have received a copy of the GNU General Public License         *
*    along with this program; if not, write to the Free Software               *
*    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA *
*******************************************************************************/

#ifndef EEPROM_H_
#define EEPROM_H_

#include <avr/eeprom.h>

// UNBEDINGT den Fuse EESAVE setzen! Ansonsten wird die EEPROM nicht einen
// Flash überleben!!!
struct eepromData
{
  bool initialised;
  unsigned int currentFileNumber;
  bool alarmOnOff;
};

#ifndef EEMEM
// alle Textstellen EEMEM im Quellcode durch __attribute__ ... ersetzen
#define EEMEM  __attribute__ ((section (".eeprom")))
#endif

#ifdef SOURCE_eeprom
// Es muss immer ein Array sein, ansonsten gibt es Probleme mit
// eeprom_write_block (will Zeiger an Array haben)
struct eepromData eeData[1] EEMEM;
struct eepromData eeWeckerData;
bool changedEEPROM;
#else
extern struct eepromData eeData[1] EEMEM;
extern struct eepromData eeWeckerData;
bool changedEEPROM;
#endif

/**
 * Holt die Daten vom EEPROM.
 * Falls es ist das erste Mal, nutzt die Defaultwerte
 */
void getDataFromEEPROM(void);

/**
 * Wenn nötig, speichert die Daten in die EEPROM
 * Diese Funktion soll ziemlich oft aufgerufen werden, um sicher zu sein,
 * dass keine Daten verloren werden.
 */
#define saveDataToEEPROM() \
{ \
  if(changedEEPROM) \
  { \
    eeprom_write_block(&eeWeckerData, &eeData[0], sizeof(eeWeckerData)); \
    changedEEPROM = FALSE; \
  } \
}

/**
 * Gibt die aktuelle Dateinummer zurück
 *
 * @return unsigned int         die Dateinummer
 */
#define getCurrentFileNumber()      eeWeckerData.currentFileNumber

/**
 * Speichert die aktuelle Dateinummer
 *
 * @param unsigned int __n      die neue Dateinummer
 */
#define setCurrentFileNumber(__n)   { eeWeckerData.currentFileNumber = __n; changedEEPROM = TRUE; }

#endif /* EEPROM_H_ */
