/*******************************************************************************
*    Wecker 1.0.                                                               *
*    Copyright (C) 2010-averyfarwaydate Luca Bertoncello                       *
*    Hartigstrasse, 12 - 01127 Dresden Deutschland                             *
*    E-Mail: lucabert@lucabert.de, lucabert@lucabert.com                       *
*    http://www.lucabert.de/  http://www.lucabert.com/                         *
*                                                                              *
*    Circuit by Luca Bertoncello and Tim Rettmann, Layout by Tim Rettmann      *
*                                                                              *
*    This program is free software; you can redistribute it and/or modify      *
*    it under the terms of the GNU General Public License as published by      *
*    the Free Software Foundation; version 2 of the License.                   *
*                                                                              *
*    This program is distributed in the hope that it will be useful,           *
*    but WITHOUT ANY WARRANTY; without even the implied warranty of            *
*    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             *
*    GNU General Public License for more details.                              *
*                                                                              *
*    You should have received a copy of the GNU General Public License         *
*    along with this program; if not, write to the Free Software               *
*    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA *
*******************************************************************************/

#ifndef MP3_H_
#define MP3_H_

#define TDA_MUTE_PORT           PORTA
#define TDA_MUTE_DDR            DDRA
#define TDA_MUTE_PIN            0

#define VS_PORT                 PORTA
#define VS_DDR                  DDRA
#define VS_PIN                  PINA
#define VS_CS_PIN               5
#define VS_RESET_PIN            4
#define VS_DREQ_PIN             3
#define VS_BSYNC_PIN            2

#define VS1001_READ             0x03
#define VS1001_WRITE            0x02

#define RESET_HARD              0
#define RESET_SOFT              1

#define SCIREGISTER_MODE        0x00
#define SCIREGISTER_CLOCKF      0x03
#define SCIREGISTER_HDAT0       0x08
#define SCIREGISTER_HDAT1       0x09
#define SCIREGISTER_VOLUME      0x0B

#ifdef SOURCE_mp3
uint8_t currentVolume;
char mp3ToPlay[33];
uint8_t mp3ToPlayDirectory;
char mp3TestFile[33];
struct fat_file_struct* fd;
#else
extern uint8_t currentVolume;
extern char mp3ToPlay[33];
extern uint8_t mp3ToPlayDirectory;
extern char mp3TestFile[33];
#endif

/**
 * Initialisiert die PINs für den VS1011e und den Verstärker
 * Beim Einschalten ist der Verstärker AUS!!
 * WICHTIG: alle Signale für den VS1011e müssen richtig gesetzt werden,
 * ansonsten bei der Initialisierung der SD-Karte wird der VS1011e auch
 * reagieren (= Müll auf dem Bus)
 */
void mp3InitHW(void);

/**
 * Nach dem erfolgreichem Konfigurationslesen aus der Karte, soll
 * der VS1011 lieber neugestartet werden
 */
#define mp3InitVS1011() \
{ \
  mp3Reset(RESET_HARD); \
}

/**
 * Schickt ein Soft-Reset dem VS1011e
 *
 * @param uint8_t type                RESET_HARD oder RESET_SOFT
 */
void mp3Reset(uint8_t type);

/**
 * Schaltet den Verstärker an
 */
#define tdaTurnOn()             TDA_ONOF_PORT &= ~_BV(TDA_ONOF_PIN);

/**
 * Schaltet den Verstärker aus
 */
#define tdaTurnOff()            TDA_ONOF_PORT |= _BV(TDA_ONOF_PIN);

/**
 * Schaltet den "Mute" des Verstärkers aus
 * (wird kein Signal dem Lautsprecher geschickt)
 */
#define tdaLoud()               TDA_MUTE_PORT &= ~_BV(TDA_MUTE_PIN);

/**
 * Schaltet den "Mute" des Verstärkers an
 * (Signale werden dem Lautsprecher geschickt)
 */
#define tdaMute()               TDA_MUTE_PORT |= _BV(TDA_MUTE_PIN);

/**
 * Setzt den Bit ChipSelect für den VS1001
 */
#define vsSetChipSelect()       VS_PORT |= _BV(VS_CS_PIN);

/**
 * Löscht den Bit ChipSelect für den VS1001
 */
#define vsClearChipSelect()     VS_PORT &= ~_BV(VS_CS_PIN);

/**
 * Setzt den Bit Reset für den VS1001
 */
#define vsSetReset()            VS_PORT |= _BV(VS_RESET_PIN);

/**
 * Löscht den Bit Reset für den VS1001
 */
#define vsClearReset()          VS_PORT &= ~_BV(VS_RESET_PIN);

/**
 * Löscht den Bit BSync für den VS1001
 */
#define vsClearBSync()          VS_PORT &= ~_BV(VS_BSYNC_PIN);

/**
 * Setzt den Bit DReq für den VS1001
 */
#define vsSetBSync()            VS_PORT |= _BV(VS_BSYNC_PIN);

/**
 * Schickt ein Byte dem VS
 *
 * @param unsigned char byte          der zu schickenden Byte
 */
void vsSendData(unsigned char byte);

/**
 * Lese zwei Bytes von einem Register
 *
 * @param uint8_t address             die Adresse des Registers
 * @return uint16_t                   die zwei Bytes als 16-Bit word
 */
uint16_t vsReadData(uint8_t address);

/**
 * Schickt zwei Bytes an einem SCI-Register der VS1011e über SPI
 *
 * @param uint8_t register            der Register
 * @param uint8_t byte1               der erste Byte zu schicken
 * @param uint8_t byte2               der zweite Byte zu schicken
 */
#define vsWrite2BytesToRegister(__register, __byte1, __byte2) \
{\
  vsClearBSync(); \
  vsClearChipSelect(); \
  spiWriteByte(VS1001_WRITE); \
  spiWriteByte(__register); \
  spiWriteByte(__byte1); \
  spiWriteByte(__byte2); \
  vsSetChipSelect(); \
}

/**
 * Setzt die Lautstärke
 *
 * @param uint8_t volume              die Lautstärke
 */
#define vsSetVolume(__volume) \
{ \
  vsWrite2BytesToRegister(SCIREGISTER_VOLUME, __volume, __volume); \
  _delay_ms(10); \
  loop_until_bit_is_set(VS_PIN, VS_DREQ_PIN); \
  _delay_ms(100); \
}

/**
 * Setzt den Clock für den VS1011e (Standardquarz: 24.576MHz).
 * Laut Doku (Seite 31) soll es immer vor dem Abspielen gesetzt werden!
 */
#define vsSetClock() \
{ \
  vsWrite2BytesToRegister(SCIREGISTER_CLOCKF, 0x00, 0x00); \
  _delay_ms(10); \
  loop_until_bit_is_set(VS_PIN, VS_DREQ_PIN); \
  _delay_ms(100); \
}

/**
 * Bereite das System für die nächste MP3.
 *
 * @param dir                         Directory der Datei (ab /)
 * @param file                        Die Datei
 */
#define setNextFileToPlay(__dir, __file)  { mp3ToPlayDirectory = __dir; strcpy(mp3ToPlay, __file); }

/**
 * Startet die Wiedergabe der Datei
 */
#define playMP3()                         { playMP3 = TRUE; }

/**
 * Gibt zurück, ob gerade eine Datei abgespielt wird
 *
 * @return                            TRUE wenn eine Datei abgespielt wird
 */
#define startedMP3()                      (playMP3 == TRUE)

/**
 * Gibt zurück, ob gerade keine Datei abgespielt wird
 *
 * @return                            TRUE wenn eine Datei abgespielt wird
 */
#define notStartedMP3()                   (playMP3 == FALSE)

/**
 * Spielt die aktuelle MP3-Datei ab
 */
void mp3Play(void);

/**
 * Holt die Dateiname der nächsten MP3
 */
void getNextMP3(void);

/**
 * Holt die Dateiname der aktuellen MP3
 */
void getActualMP3(void);

/**
 * Holt die MP3-Tags (Titel, Komponist, Kommentare) wenn sie geben.
 * Speichert das ganze in der ScrollString.
 * Wenn keine MP3-Tags gibt, wird den Dateiname gezeigt
 *
 * Zuerst wird versucht die Tags nach ID3v2 zu lesen.
 * Wenn kein ID3v2-Tag gibt, dann wird versucht die Tags nach ID3v1
 * zu lesen.
 * Fehlen auch diese Tags, wird den Dateiname angezeigt
 */
void mp3GetTags(void);

/**
 * Liest einen ID3v2-Header (10 Bytes)
 *
 * @param struct fat_file_struct* fd  der Filedescriptor
 * @param char* buf                   die String, wo der ID3v2-Header gespeichert wird
 */
#define getID3(__fd, __buf)               { fat_read_file(__fd, __buf, 10); }

/**
 * Speichert die Directory, wo die MP3 liegen
 *
 * @param char *__dir                 Die Directory
 */
#define setMP3Directory(__dir)            { strcpy(mp3Directory, __dir); }

#endif /* MP3_H_ */
