/*******************************************************************************
*    Wecker 1.0.                                                               *
*    Copyright (C) 2010-averyfarwaydate Luca Bertoncello                       *
*    Hartigstrasse, 12 - 01127 Dresden Deutschland                             *
*    E-Mail: lucabert@lucabert.de, lucabert@lucabert.com                       *
*    http://www.lucabert.de/  http://www.lucabert.com/                         *
*                                                                              *
*    Circuit by Luca Bertoncello and Tim Rettmann, Layout by Tim Rettmann      *
*                                                                              *
*    This program is free software; you can redistribute it and/or modify      *
*    it under the terms of the GNU General Public License as published by      *
*    the Free Software Foundation; version 2 of the License.                   *
*                                                                              *
*    This program is distributed in the hope that it will be useful,           *
*    but WITHOUT ANY WARRANTY; without even the implied warranty of            *
*    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             *
*    GNU General Public License for more details.                              *
*                                                                              *
*    You should have received a copy of the GNU General Public License         *
*    along with this program; if not, write to the Free Software               *
*    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA *
*******************************************************************************/

#ifndef SPI_H_
#define SPI_H_

#include <avr/io.h>

#define MOSI                    PB5
#define MISO                    PB6
#define SCK                     PB7
#define SPI_PORT                PORTB
#define SPI_DDR                 DDRB

/**
 * Schickt ein Byte über SPI
 *
 * @param uint8_t byte        der Byte zu schicken
 */
#define spiWriteByte(__byte)      { SPDR = __byte; asm volatile("nop"); loop_until_bit_is_set(SPSR, SPIF); }

/*
 * Setzt die Geschwindigkeit des SPI-Clocks auf µC-Clock/2 (8MHz)
 * So kann man die SD-Karte schnell ansprechen
 */
#define spiSetHighSpeed() \
{ \
  SPCR &= ~((1 << SPR1) | (1 << SPR0)); \
  SPSR |= (1 << SPI2X); \
}

/**
 * Setzt die Geschwindigkeit des SPI-Clocks auf µC-Clock/8 (2MHz)
 * Das ist NOTWENDIG um mit dem VS1011e zu sprechen, denn er kann
 * einen maximalen SPI-Clock von VS-Clock/7 (~3.5MHz) verstehen!
 */
#define spiSetLowSpeed() \
{ \
  SPCR &= ~(1 << SPR1); \
  SPCR |= (1 << SPR0); \
  SPSR |= (1 << SPI2X); \
}

#endif /* SPI_H_ */
