/*******************************************************************************
*    Wecker 1.0.                                                               *
*    Copyright (C) 2010-averyfarwaydate Luca Bertoncello                       *
*    Hartigstrasse, 12 - 01127 Dresden Deutschland                             *
*    E-Mail: lucabert@lucabert.de, lucabert@lucabert.com                       *
*    http://www.lucabert.de/  http://www.lucabert.com/                         *
*                                                                              *
*    Circuit by Luca Bertoncello and Tim Rettmann, Layout by Tim Rettmann      *
*                                                                              *
*    This program is free software; you can redistribute it and/or modify      *
*    it under the terms of the GNU General Public License as published by      *
*    the Free Software Foundation; version 2 of the License.                   *
*                                                                              *
*    This program is distributed in the hope that it will be useful,           *
*    but WITHOUT ANY WARRANTY; without even the implied warranty of            *
*    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             *
*    GNU General Public License for more details.                              *
*                                                                              *
*    You should have received a copy of the GNU General Public License         *
*    along with this program; if not, write to the Free Software               *
*    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA *
*******************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <avr/interrupt.h>
#include <util/delay.h>
#include "inc/wecker.h"
#include "inc/eeprom.h"
#include "inc/tools.h"
#include "inc/dogm.h"
#include "inc/tools.h"
#include "inc/dcf77.h"
#include "inc/time.h"
#include "inc/button.h"
#include "inc/mp3.h"
#include "inc/spi.h"
#include "inc/card.h"
#include "inc/alarm.h"
#include "resources/dict.h"

int main(void)
{
  bool first = TRUE;

  displayErrorTime = 0L;
  lcdInit(POWER3V3, 11, 0, 0);
  dcf77Init();
  dcf77InitTimer();
  initButtons();
  MMCALLOW_DDR |= _BV(MMCALLOW_PIN);
  sei();
  mp3InitHW();

  enableMMC();
  initCard(TRUE);
  readConfig(FALSE);
  disableMMC();

  _delay_ms(100);
  mp3InitVS1011();

  defineAntenna();
  defineClock();
  lcdClear();
  initAlarm();
  getDataFromEEPROM();

// Gerade eingeschaltet, also lesen wir die Zeit
#ifndef DEVELOP
  if(dcf77GetTime(first) == TRUE)
  { // die Weckzeit wird nur gelesen, wenn die Zeit synchron ist
    first = FALSE;
    getNextAlarm(TRUE);
  }
#endif

  while(1)
  {
    displayCurrentTime();
    saveDataToEEPROM();
    if(startedMP3())
      mp3Play();
    else
    {
      if(sd_raw_available())
      {
        if(currentTime.seconds == 0 && hasToStartAlarm())
        {
          if(isAlarmOn())
          {
            if(alarmFile[0] == 0)
              getNextMP3();
            else
              setNextFileToPlay(MP3DIREXTRA, alarmFile);
            playMP3();
          }
          hasToCalculateNextAlarm = TRUE;
        }
        if(hasToCalculateNextAlarm && notStartedMP3() && (displayErrorTime == 0L))
        {
          getNextAlarm(FALSE);
          hasToCalculateNextAlarm = FALSE;
        }
      }
      if(displayErrorTime != 0L && (getUptime() > (displayErrorTime + MAXDISPLAYERROR)))
      {
        displayErrorTime = 0L;
        stopScroll();
        lcdBacklightOff();
      }
      if(startGetNextMP3 != 0L && (getUptime() > (startGetNextMP3 + maxDisplayNextMP3)))
      {
        startGetNextMP3 = 0L;
        stopScroll();
        lcdBacklightOff();
      }
    }
// Synchronisierung der Zeit. Entweder automatisch berechnet (siehe displayCurrentTime) oder nach Knopfdruck
// Ausnahme ist, wenn gerade ein MP3 gespielt wird.
// Diese Funktion wird am Ende der Schleife aufgerufen, denn ansonsten, wenn
// die Weckzeit gerade bei der Synchronisierung wäre, würde den Wecker nicht
// abspielen.
    if(hasToStartSync() && !startedMP3())
    {
      if((dcf77GetTime(first) == TRUE) && (first == TRUE))
      { // Wenn die Zeit zum ersten Mal synchron ist, wird die nächste Weckzeit gelesen
        first = FALSE;
        getNextAlarm(TRUE);
      }
    }
  }

  return 0;
}
