/*******************************************************************************
*    Wecker 1.0.                                                               *
*    Copyright (C) 2010-averyfarwaydate Luca Bertoncello                       *
*    Hartigstrasse, 12 - 01127 Dresden Deutschland                             *
*    E-Mail: lucabert@lucabert.de, lucabert@lucabert.com                       *
*    http://www.lucabert.de/  http://www.lucabert.com/                         *
*                                                                              *
*    Circuit by Luca Bertoncello and Tim Rettmann, Layout by Tim Rettmann      *
*                                                                              *
*    This program is free software; you can redistribute it and/or modify      *
*    it under the terms of the GNU General Public License as published by      *
*    the Free Software Foundation; version 2 of the License.                   *
*                                                                              *
*    This program is distributed in the hope that it will be useful,           *
*    but WITHOUT ANY WARRANTY; without even the implied warranty of            *
*    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             *
*    GNU General Public License for more details.                              *
*                                                                              *
*    You should have received a copy of the GNU General Public License         *
*    along with this program; if not, write to the Free Software               *
*    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA *
*******************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <avr/interrupt.h>
#include <util/delay.h>
#include "inc/wecker.h"
#include "inc/eeprom.h"
#include "inc/tools.h"
#include "inc/button.h"
#include "inc/mp3.h"
#include "inc/spi.h"
#include "inc/dcf77.h"
#include "inc/time.h"
#include "inc/dogm.h"
#include "inc/card.h"
#include "resources/dict.h"

/**
 * Initialisiert die PINs für den VS1011e und den Verstärker
 * Beim Einschalten ist der Verstärker AUS!!
 * WICHTIG: alle Signale für den VS1011e müssen richtig gesetzt werden,
 * ansonsten bei der Initialisierung der SD-Karte wird der VS1011e auch
 * reagieren (= Müll auf dem Bus)
 */
void mp3InitHW()
{
  TDA_MUTE_DDR |= _BV(TDA_MUTE_PIN);

  VS_DDR |= _BV(VS_CS_PIN);
  VS_DDR |= _BV(VS_RESET_PIN);
  VS_DDR &= ~_BV(VS_DREQ_PIN);
  VS_PORT |= _BV(VS_DREQ_PIN);
  VS_DDR |= _BV(VS_BSYNC_PIN);

// Defaults, falls auf der Karte nicht gespeichert
  strcpy(mp3TestFile, MP3DEFAULTTESTFILE);
  currentVolume = STARTVOLUME;
  bzero(mp3ToPlay, 32);
  mp3ToPlayDirectory = MP3DIRNORMAL;

// OK, alles initialisiert, Verstärker in Mute halten
  tdaMute();

// Unbedingt so setzen, ansonsten bei der Initialisierung der SD-Karte
// wird der VS1011e auch reagieren und wird nur Müll auf den Bus kommen
  vsSetChipSelect();
  vsSetReset();
  vsClearBSync();
}

/**
 * Schickt ein Soft-Reset dem VS1011e
 *
 * @param uint8_t type          RESET_HARD oder RESET_SOFT
 */
void mp3Reset(uint8_t type)
{
  uint16_t i;

  if(type == RESET_HARD)
  {
    vsClearReset();
    _delay_ms(100);
    SPDR = 0xFF;
    vsSetChipSelect();
    vsSetBSync();
    vsSetReset();
    _delay_ms(100);
  }

// Auf alle Fälle ein SoftReset. Es ist auch explizit verlangt nach dem HardReset
  vsWrite2BytesToRegister(SCIREGISTER_MODE, 0x00, 0x04);
  _delay_ms(10);
  loop_until_bit_is_set(VS_PIN, VS_DREQ_PIN);
  _delay_ms(100);
// Und Clock setzen, noch
  vsSetClock();

// Stumme Daten, so daß der VS1011e nicht Mist abspielt
// Eigentlich sollte es nicht nötig sein, aber sicher ist sicher
  for(i = 0; i < 2048;)
  {
    if(bit_is_set(VS_PIN, VS_DREQ_PIN))
    {
      vsSendData(0);
      i++;
    }
    displayCurrentTime();
  }
}

/**
 * Schickt ein Byte dem VS
 *
 * @param unsigned char byte    der zu schickenden Byte
 */
void vsSendData(unsigned char byte)
{
  vsSetBSync();

  SPDR = byte;

  asm volatile("nop");
  asm volatile("nop");
  asm volatile("nop");

  vsClearBSync();

  loop_until_bit_is_set(SPSR, SPIF);
}

/**
 * Lese zwei Bytes von einem Register
 *
 * @param uint8_t address       die Adresse des Registers
 * @return uint16_t             die zwei Bytes als 16-Bit word
 */
uint16_t vsReadData(uint8_t address)
{
  uint16_t r;

  vsClearChipSelect();
  spiWriteByte(VS1001_READ);
  spiWriteByte(address);
  spiWriteByte(0xFF);
  r = SPDR << 8;
  spiWriteByte(0xFF);
  r |= SPDR;
  vsSetChipSelect();

  return(r);
}

/**
 * Spielt die aktuelle MP3-Datei ab
 */
void mp3Play()
{
  uint16_t i, d1, d2, r;
  unsigned char mp3[100]; // 3 Pakete für den VS1011e
  char err[50];

  spiSetHighSpeed();
  enableMMC();
  chDir("/");
  bzero(err, 50);
  strcpy(err, ERROROPENFILE);
  strcat(err, " [/");
  switch(mp3ToPlayDirectory)
  {
    case MP3DIRNORMAL:
      chDir(MP3DIRECTORY);
      strcat(err, MP3DIRECTORY);
    break;
    case MP3DIREXTRA:
      chDir(MP3EXTRADIRECTORY);
      strcat(err, MP3EXTRADIRECTORY);
    break;
  }
  strcat(err, "/");
  strcat(err, mp3ToPlay);
  strcat(err, "]");
  fd = fopen(mp3ToPlay);
  if(!fd)
  {
    displayError(err);
    playMP3 = FALSE;
    return;
  }
  mp3GetTags();
  setScrollLine(2);
  startScroll();
  lcdBacklightOn();
  spiSetLowSpeed();

// Laut Doku, soll man den vor dem Abspielen immer setzen!
  vsSetClock();
// Dazu setze ich die Lautstärke, denn ansonsten (bei Knopfbetätigen) wird nicht gesetzt!
  vsSetVolume(currentVolume);
  tdaLoud();

  while(!fat_feof(fd))
  {
    bzero(mp3, 96); // Wenn man einen Puffer von 3 Pakete (je 32 Bytes) liest, ist das Abspielen besser
// Beim Ablesen der MP3-Daten soll eine höhere Geschwindigkeit benutzt werden,
// dann kann es nicht passieren, daß das Abspielen abbricht
    spiSetHighSpeed();
    r = fat_read_file(fd, mp3, 96);
    spiSetLowSpeed();
    if(r != -1)
    {
      vsSetChipSelect();
      for(i = 0; i < r;)
        if(bit_is_set(VS_PIN, VS_DREQ_PIN))
          vsSendData(mp3[i++]);
    }
    displayCurrentTime();
  }
  fclose(fd);
  disableMMC();

// So daß das Abspielen gut bis zum Ende geht...
  vsSetChipSelect();
  for(i = 0; i < 2048;)
  {
    if(bit_is_set(VS_PIN, VS_DREQ_PIN))
    {
      vsSendData(0);
      i++;
    }
    displayCurrentTime();
  }
  _delay_ms(100);

  d1 = d2 = 1;
  do
  {
    d1 = vsReadData(SCIREGISTER_HDAT0);
    d2 = vsReadData(SCIREGISTER_HDAT1);
    displayCurrentTime();
  }
  while((d1 != 0) && (d2 != 0));
  vsSetChipSelect();
  tdaMute();
  playMP3 = FALSE;
  stopScroll();
  lcdBacklightOff();
}

/**
 * Holt die Dateiname der nächsten MP3
 */
void getNextMP3()
{
  struct fat_dir_entry_struct dir_entry;
  bool found = FALSE;
  char fileName[33];
  uint8_t l;

  spiSetHighSpeed();
  enableMMC();
  chDir("/");
  chDir(MP3DIRECTORY);

  l = 0;
  while(fat_read_dir(dd, &dir_entry) && (found == FALSE))
  {
    if(dir_entry.long_name[0] != '.')
    {
      if(l == getCurrentFileNumber())
      {
        strncpy(fileName, dir_entry.long_name, 31);
        found = TRUE;
        break;
      }
      l++;
    }
  }
  if(!found)
  { // Ende der Directory. Wieder die erste Datei holen
    setCurrentFileNumber(0);
    while(!found)
    {
      fat_read_dir(dd, &dir_entry);
      if(dir_entry.long_name[0] != '.')
      {
        strncpy(fileName, dir_entry.long_name, 31);
        found = TRUE;
      }
    }
  }
  disableMMC();
  spiSetLowSpeed();

  setCurrentFileNumber(getCurrentFileNumber() + 1);
  setNextFileToPlay(MP3DIRNORMAL, fileName);
}

/**
 * Holt die Dateiname der aktuellen MP3
 */
void getActualMP3()
{
  struct fat_dir_entry_struct dir_entry;
  bool found = FALSE;
  char fileName[33];
  uint8_t l;

  spiSetHighSpeed();
  enableMMC();
  chDir("/");
  chDir(MP3DIRECTORY);

  l = 0;
  while(fat_read_dir(dd, &dir_entry) && (found == FALSE))
  {
    if(dir_entry.long_name[0] != '.')
    {
      if(l == getCurrentFileNumber())
      {
        strncpy(fileName, dir_entry.long_name, 31);
        found = TRUE;
        break;
      }
      l++;
    }
  }
  disableMMC();
  spiSetLowSpeed();

  setNextFileToPlay(MP3DIRNORMAL, fileName);
}

/**
 * Holt die MP3-Tags (Titel, Komponist, Kommentare) wenn sie geben.
 * Speichert das ganze in der ScrollString.
 * Wenn keine MP3-Tags gibt, wird den Dateiname gezeigt
 *
 * Zuerst wird versucht die Tags nach ID3v2 zu lesen.
 * Wenn kein ID3v2-Tag gibt, dann wird versucht die Tags nach ID3v1
 * zu lesen.
 * Fehlen auch diese Tags, wird den Dateiname angezeigt
 */
void mp3GetTags()
{
  int32_t offset = -128;
  uint8_t l, i, title, artist, comment, ao, to, co, *o;
  int16_t len;
  uint32_t size, cur, tagSize, p;
  unsigned char tags[11];
  char str[MAXSCROLLSTRING + 1];
  unsigned char ch[2];

  setStringToScroll(mp3ToPlay);
  bzero(str, MAXSCROLLSTRING + 1);

// Erstmal wird versucht, die Tags nach ID3v2 zu lesen
  getID3(fd, tags);
  if(tags[0] == 'I' &&
     tags[1] == 'D' &&
     tags[2] == '3')
  {
    size = (uint32_t)(((unsigned char)tags[6]) & 0x7F) << 21 |
           (uint32_t)(((unsigned char)tags[7]) & 0x7F) << 14 |
           (uint32_t)(((unsigned char)tags[8]) & 0x7F) << 7  |
           (uint32_t)(((unsigned char)tags[9]) & 0x7F);
    cur = 0;
    artist = title = comment = 0;
    o = &ao;
    while(cur < size)
    {
      getID3(fd, tags);
      cur += 10;
      if(tags[0] != 0)
      {
        tagSize = ((unsigned char)tags[4]) |
                  ((unsigned char)tags[5]) |
                  ((unsigned char)tags[6]) |
                  ((unsigned char)tags[7]);
        cur += tagSize;
        offset = -1;
        if(tags[0] == 'T' &&
           tags[1] == 'P' &&
           tags[2] == 'E' &&
           tags[3] == '1')
        {
          offset = 0;
          artist = 1;
          o = &ao;
        }
        if(tags[0] == 'T' &&
           tags[1] == 'I' &&
           tags[2] == 'T' &&
           tags[3] == '2')
        {
          offset = 51;
          title = 1;
          o = &to;
        }
        if(tags[0] == 'C' &&
           tags[1] == 'O' &&
           tags[2] == 'M' &&
           tags[3] == 'M')
        {
          offset = 102;
          comment = 1;
          o = &co;
        }
        if(offset != -1)
        {
          for(l = 0; l < 48; l++)
            str[offset + l] = 0;
          *(o) = 0;
          for(p = l = 0; l < tagSize; l++)
          {
            fat_read_file(fd, ch, 1);
            if(ch[0] != 0)
              if((offset + p) < MAXSCROLLSTRING)
              {
                str[offset + p++] = ch[0];
                (*o)++;
              }
          }
        }
        else
          for(l = 0; l < tagSize; l++)
            fat_read_file(fd, ch, 1);
      }
      else
      {
        ch[0] = 0;
        while(ch[0] == 0)
        {
          fat_read_file(fd, ch, 1);
          if(ch[0] == 0)
            cur++;
        }
      }
    }

// Alle Informationen zusammenfassen
    if(title == 1 || artist == 1 || comment == 1)
    {
      p = 0;
      if(title == 1)
      {
        str[ao++] = ' ';
        str[ao++] = '-';
        str[ao++] = ' ';
        for(p = ao, l = 0; l < to; l++, p++)
          str[p] = str[51 + l];
      }
      if(comment == 1)
      {
        str[p++] = ' ';
        str[p++] = '-';
        str[p++] = ' ';
        for(l = 0; l < co; l++, p++)
          str[p] = str[102 + l];
      }
      str[p] = 0;

      setStringToScroll(str);
      return;
    }
  }

  // Keine ID3v2 Informationen gefunden. Es wird nach ID3v1 gesucht
  if(fat_seek_file(fd, &offset, FAT_SEEK_END) != 0)
  {
    if((len = fat_read_file(fd, tags, 3)) == 3)
    {
      if(tags[0] == 'T' &&
         tags[1] == 'A' &&
         tags[2] == 'G')
      {
        i = 0;
        for(l = 0; l < 30 && ch[l] != 0; l++)        // Title
        {
          fat_read_file(fd, ch, 1);
          str[i++] = ch[0];
        }
        for(l = 0; l < 30 && ch[l] != 0; l++)        // Artist
        {
          fat_read_file(fd, ch, 1);
          str[i++] = ch[0];
        }
        for(l = 0; l < 34; l++)                      // Album, Year
          fat_read_file(fd, ch, 1);
        for(l = 0; l < 30 && ch[l] != 0; l++)        // Comment
        {
          fat_read_file(fd, ch, 1);
          str[i++] = ch[0];
        }
        setStringToScroll(str);
      }
    }
    offset = 0;
    fat_seek_file(fd, &offset, FAT_SEEK_SET);
  }
}
