/*******************************************************************************
*    Wecker 1.0.                                                               *
*    Copyright (C) 2010-averyfarwaydate Luca Bertoncello                       *
*    Hartigstrasse, 12 - 01127 Dresden Deutschland                             *
*    E-Mail: lucabert@lucabert.de, lucabert@lucabert.com                       *
*    http://www.lucabert.de/  http://www.lucabert.com/                         *
*                                                                              *
*    Circuit by Luca Bertoncello and Tim Rettmann, Layout by Tim Rettmann      *
*                                                                              *
*    This program is free software; you can redistribute it and/or modify      *
*    it under the terms of the GNU General Public License as published by      *
*    the Free Software Foundation; version 2 of the License.                   *
*                                                                              *
*    This program is distributed in the hope that it will be useful,           *
*    but WITHOUT ANY WARRANTY; without even the implied warranty of            *
*    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             *
*    GNU General Public License for more details.                              *
*                                                                              *
*    You should have received a copy of the GNU General Public License         *
*    along with this program; if not, write to the Free Software               *
*    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA *
*******************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include "inc/wecker.h"
#include "inc/eeprom.h"
#include "inc/dogm.h"
#include "inc/dcf77.h"
#include "inc/time.h"
#include "inc/mp3.h"
#include "inc/button.h"
#include "inc/card.h"
#include "inc/alarm.h"
#include "resources/dict.h"

/**
 * Definiert den Zeichen für die Antenne
 */
void defineAntenna()
{
  lcdSendInstruction(0x48);
  lcdPutc(21);
  lcdPutc(10);
  lcdPutc(10);
  lcdPutc(4);
  lcdPutc(4);
  lcdPutc(4);
  lcdPutc(14);
  lcdPutc(14);
}

/**
 * Definiert den Zeichen für den Wecker
 */
void defineClock()
{
  lcdSendInstruction(0x50);
  lcdPutc(14);
  lcdPutc(21);
  lcdPutc(21);
  lcdPutc(23);
  lcdPutc(17);
  lcdPutc(17);
  lcdPutc(14);
  lcdPutc(0);
}

/**
 * Zeigt die aktuelle Zeit und Datum
 * auf dem Display
 */
void displayCurrentTime()
{
  char str[5];
  char wd[7][4] = {SUNDAY, MONDAY, TUESDAY, WEDNESDAY, THURSDAY, FRIDAY, SATURDAY};

// Alle X (1 oder 4, je nachdem) Stunden wird die Zeit neu synchronisiert.
#ifndef DEVELOP
  if(currentTime.seconds == 0 && currentTime.minutes == 0 && (currentTime.hours % hourToSync) == 0)
    startSync();
#endif
  manageButtons();
  if((startDisplayVolume == 0L || startGetNextMP3 != 0L) && hasToScroll())
    doScroll();
  if(!startedMP3())
    manageCard();
  if(startDisplayVolume != 0L && (getUptime() > (startDisplayVolume + DISPLAYVOLUMETIME)))
  {
    countMenuPress = 0;
    startDisplayVolume = 0L;
    lcdGotoXY(0, 2);
    lcdPuts(CLEARLINE);
  }

  if(displayMenu == 0L && displayLastSync == 0L && displayNextAlarm == 0L && startGetNextMP3 == 0L)
  { // Es wird die normale Zeit angezeigt
    if(hasToDisplay())
    {
      lcdGotoXY(0, 0);
      if(isAlarmOn() && sd_raw_available())
      {
        if(isHoliday && (currentTime.seconds % 2))
          lcdPutc('*');
        else
          lcdPutc(CLOCKCHARCODE);
      }
      else
        lcdPutc(' ');
      lcdPutc(' ');
      lcdPutc(' ');
      lcdPutc(' ');
      if(currentTime.hours < 10)
        lcdPutc('0');
      itoa(currentTime.hours, str, 10);
      lcdPuts(str);
      lcdPutc(':');
      if(currentTime.minutes < 10)
        lcdPutc('0');
      itoa(currentTime.minutes, str, 10);
      lcdPuts(str);
      lcdPutc(':');
      if(currentTime.seconds < 10)
        lcdPutc('0');
      itoa(currentTime.seconds, str, 10);
      lcdPuts(str);
      lcdPutc(' ');
      lcdPutc(' ');
      lcdPutc(' ');
      if(valid == 1)
      {
        lcdGotoXY(15, 0);
        lcdPutc(ANTENNACHARCODE);
      }
      else
      {
        lcdGotoXY(15, 0);
        lcdPutc(' ');
      }

      lcdGotoXY(0, 1);
      lcdPutc(' ');
      lcdPuts(wd[currentTime.weekDay]);
      lcdPutc(' ');
      if(currentTime.day < 10)
        lcdPutc('0');
      itoa(currentTime.day, str, 10);
      lcdPuts(str);
      lcdPutc('.');
      if(currentTime.month < 10)
        lcdPutc('0');
      itoa(currentTime.month, str, 10);
      lcdPuts(str);
      lcdPutc('.');
      itoa(currentTime.year + 2000, str, 10);
      lcdPuts(str);
      lcdPutc(' ');
      clearToDisplay();
    }
    return;
  }

  if(displayMenu != 0L)
  { // Zeigt die Informationen über das Programm
    lcdGotoXY(0, 0);
    lcdPuts(INFO1);
    lcdGotoXY(0, 1);
    lcdPuts("  ");
    lcdPuts(VERSION);
    lcdPuts("  ");
    lcdGotoXY(0, 2);
    lcdPuts(INFO2);
  }
  if(displayLastSync != 0L)
  { // Es wird die Zeit der letzten Synchronisierung angezeigt
    lcdGotoXY(0, 0);
    lcdPuts(LASTSYNC);
    lcdGotoXY(0, 1);
    if(lastSyncTime.day < 10)
      lcdPutc('0');
    itoa(lastSyncTime.day, str, 10);
    lcdPuts(str);
    lcdPutc('.');
    if(lastSyncTime.month < 10)
      lcdPutc('0');
    itoa(lastSyncTime.month, str, 10);
    lcdPuts(str);
    lcdPutc('.');
    itoa(lastSyncTime.year + 2000, str, 10);
    lcdPuts(str);
    lcdPutc(' ');
    if(lastSyncTime.hours < 10)
      lcdPutc('0');
    itoa(lastSyncTime.hours, str, 10);
    lcdPuts(str);
    lcdPutc(':');
    if(lastSyncTime.minutes < 10)
      lcdPutc('0');
    itoa(lastSyncTime.minutes, str, 10);
    lcdPuts(str);
    lcdGotoXY(0, 2);
    lcdPuts(CLEARLINE);
  }
  if(displayNextAlarm != 0L)
  { // Es wird die nächste Weckzeit angezeigt
    lcdGotoXY(0, 0);
    lcdPuts(NEXTALARM);
    lcdGotoXY(0, 1);
    if(alarmTime.day < 10)
      lcdPutc('0');
    itoa(alarmTime.day, str, 10);
    lcdPuts(str);
    lcdPutc('.');
    if(alarmTime.month < 10)
      lcdPutc('0');
    itoa(alarmTime.month, str, 10);
    lcdPuts(str);
    lcdPutc('.');
    itoa(alarmTime.year + 2000, str, 10);
    lcdPuts(str);
    lcdPutc(' ');
    if(alarmTime.hours < 10)
      lcdPutc('0');
    itoa(alarmTime.hours, str, 10);
    lcdPuts(str);
    lcdPutc(':');
    if(alarmTime.minutes < 10)
      lcdPutc('0');
    itoa(alarmTime.minutes, str, 10);
    lcdPuts(str);
    lcdGotoXY(0, 2);
    if(displayHoliday)
    {
      if(isHoliday)
        lcdPuts(RESETHOLIDAY);
      else
        lcdPuts(SETHOLIDAY);
    }
    else
      lcdPuts(CHANGEALARM);
  }
}
